class Backend(object):
    # should be implemented all methods
    def set(self, name, value):
        raise NotImplementedError()

    def get(self, name):
        raise NotImplementedError()

    def delete(self, name):
        raise NotImplementedError()

    def set_fields(self):
        raise NotImplementedError()

    def fields(self):
        raise NotImplementedError()

    def values(self):
        raise NotImplementedError()


class AbstractBackend(Backend):
    bucket_prefix = 'ONTHEFLY'

    def __init__(self, options, original=None):
        self.options = options
        self.original = original
        self.field_registry = None
        self.value_registry = {}

    def get_value_from_original_settings(self, name):
        return getattr(self.original, name)

    def get_fields(self):
        if self.field_registry is None:
            self.field_registry = self.fields()
        return self.field_registry

    def add_field(self, name):
        if name not in self.get_fields:
            self.field_registry.append(name)
            self.set_fields()

    def delete_field(self, name):
        if name in self.field_registry:
            self.field_registry.remove(name)
            self.set_fields()

    def set_value(self, name, value):
        if name in self.value_registry:
            del self.value_registry[name]
        return self.set(name, value)

    def get_value(self, name):
        if name in self.value_registry:
            value = self.value_registry[name]
        else:
            value = self.get(name)
            self.value_registry[name] = value
        return value

    def delete_value(self, name):
        if name in self.value_registry:
            del self.value_registry[name]
        return self.delete(name)



import json
from django import template

register = template.Library()


@register.filter(name='jsondumps')
def jsondumps(value):
    if type(value) in [dict, list]:
        return json.dumps(value)
    return value



import os
import imp
import json

SUPPORTED_TYPES = [int, float, str, bool, dict, list]


def load_class(path):
    """
    Loads class from path.
    """

    mod_name, klass_name = path.rsplit('.', 1)
    mod_args = mod_name.split('.')
    mod_args.pop(0)

    try:
        import onthefly as djonthefly
        mod_path = '%s/%s/%s.py' % (
            os.path.dirname(djonthefly.__file__), mod_args[0], mod_args[1])
        mod = imp.load_source(mod_name, mod_path)
    except AttributeError as e:
        raise ImproperlyConfigured(
            'Error importing {0}: "{1}"'.format(mod_name, e))

    try:
        klass = getattr(mod, klass_name)
    except AttributeError:
        raise ImproperlyConfigured(
            'Module "{0}" does not define a "{1}" class'.format(mod_name,
                                                                klass_name))

    return klass


def convert(value, _type):
    # convert builtin types: int,str,unicode,float,bool,dict,list
    types = {
        str: lambda x: x,
        unicode: lambda x: str(x),
        bool: lambda x: x == 'True',
        list: lambda x: json.loads(x),
        dict: lambda x: json.loads(x),
        int: lambda x: int(x),
        float: lambda x: float(x),
    }
    cast_func = types.get(_type)
    return cast_func(value) if cast_func else None



from django.contrib import admin
from django.views.generic import TemplateView
from django.conf import settings
from django.contrib import messages
from onthefly.utils import SUPPORTED_TYPES, convert


class AppSettingsView(TemplateView):
    """
    Show current settings
    """
    template_name = 'admin/onthefly_settings.html'

    def get_context_data(self, *args, **kwargs):
        context = super(AppSettingsView, self).get_context_data(
            *args, **kwargs)
        context.update({
            'original_settings': (
                settings.get_original_settings_without_onthefly),
            'onthefly_settings': settings.get_onthefly_settings
        })
        return context

    def post(self, request, *args, **kwargs):
        action_type = request.POST.get('actiontype')
        name = request.POST.get('name')
        if action_type == 'add_field':
            original_value = settings.backend.get_value_from_original_settings(
                name)
            if original_value is None:
                messages.error(
                    request, 'NoneType objects can not be changed at runtime!')
            elif type(original_value) not in SUPPORTED_TYPES:
                messages.error(
                    request,
                    '%s is not supported to add ONTHEFLY settings!' % (
                        type(original_value)), )
            else:
                settings.backend.add_field(name)
                settings.backend.set_value(name, original_value)
        elif action_type == 'delete_field':
            settings.backend.delete_field(name)
            settings.backend.delete_value(name)
        elif action_type == 'set_value':
            original_value = settings.backend.get_value_from_original_settings(
                name)
            if type(original_value) not in SUPPORTED_TYPES:
                messages.error(
                    request,
                    '%s is not supported to add ONTHEFLY settings!' % (
                        type(original_value)), )
            value = request.POST.get('value')
            converted_value = convert(value, type(original_value))
            if converted_value is None:
                messages.error(
                    request,
                    'Original value type is different than current type!')
            else:
                settings.backend.set_value(name, converted_value)
        request.method = 'GET'
        return self.get(request, *args, **kwargs)


admin.site.register_view('onthefly-settings/', 'Onthefly Settings',
                         view=AppSettingsView.as_view())
